/*
	file:  AddonApp.cpp
	License: Public Domain
	Creator: Jonas Sundström, www.Kirilla.com
	Credits: John R. Ashmun, Tracker AddOn Test Base
*/

#include <Application.h>
#include <Roster.h>
#include <String.h>
#include <unistd.h>

#include "AddonApp.h"

const char * signature = "application/x-vnd.kirilla-trackeraddonshellinterface";

AddonApp::AddonApp	(void)	:	BApplication (signature)
{
	// void
}


AddonApp::~AddonApp (void)
{
	// void
}


typedef void	(*process_refs_func)	(entry_ref, BMessage*, void*);


void AddonApp::ArgvReceived (int32 argc, char **argv)
{
	// init
	status_t		status	=	B_OK;
	image_id		id		=	0;		// id for addon memory image

	// pointer to addons process_refs hook function 
	process_refs_func	current_process_refs = 0;

	// get current working dir as dir_ref for addon 
	BString			current_dir;
	getcwd(current_dir.LockBuffer(4096), 4096);
	current_dir.UnlockBuffer();

	BEntry			dir_entry(current_dir.String());
	entry_ref		dir_ref;
	dir_entry.GetRef(& dir_ref);

	// create message to pass to addon 
	BMessage	*	msg	=	new	BMessage(B_REFS_RECEIVED);

	int x	=	2;
	for (x = 2; x < argc; x++)
	{
		printf("argv[%d]: %s\n", x, argv[x]);	// addon to run
		BEntry	entry	(argv[x]);
		entry_ref	ref;
		entry.GetRef(& ref);
		msg->AddRef("refs", & ref);
	} 

	// load Tracker addon image
	id = load_add_on	(argv[1]);

	if	(id < 0)
	{
		printf( "Couldn't load addon: \"%s\"\n", argv[1]);
		return;
	}

	// get addon hook function (process_refs)
	status = get_image_symbol (id, "process_refs", B_SYMBOL_TYPE_TEXT, 
								(void**)&current_process_refs );

	if( status != B_OK )
	{
		printf ("Loaded addon \"%s\",\n"
				"but couldn't find process_refs().\n"
				"Probably not a Tracker addon.\n", argv[1]);
		return;
	}

	// call hook function 
	current_process_refs	(dir_ref, msg, 0);

	// unload addon
	unload_add_on	(id);
}

void AddonApp::ReadyToRun(void)
{
	be_app->PostMessage( B_QUIT_REQUESTED );
}
